# SPDX-License-Identifier: Apache-2.0

"""Common customer-related models used across ISO20022 messages."""

from dataclasses import dataclass
from typing import List, Optional

from miso20022.common.account import IdAcct


@dataclass
class PstlAdr:
    """Postal address information."""
    StrtNm: Optional[str] = None
    BldgNb: Optional[str] = None
    BldgNm: Optional[str] = None  # Added to handle Building Name field
    Flr: Optional[str] = None
    Room: Optional[str] = None
    PstCd: Optional[str] = None
    PstBx: Optional[str] = None  # Added to handle PostBox field
    TwnNm: Optional[str] = None
    TwnLctnNm: Optional[str] = None  # Added to handle Town Location Name field
    DstrctNm: Optional[str] = None  # Added to handle District Name field
    CtrySubDvsn: Optional[str] = None
    Ctry: Optional[str] = None
    AdrLine: Optional[List[str]] = None
    Dept: Optional[str] = None  # Added to handle Department field
    SubDept: Optional[str] = None  # Added to handle SubDepartment field

@dataclass
class ClrSysId:
    """Clearing system identification."""
    Cd: str = "USABA"
    Agt: Optional[Agt] = None

@dataclass
class ClrSysMmbId:
    """Clearing system member identification."""
    ClrSysId: ClrSysId
    MmbId: str


@dataclass
class FinInstnId:
    """Financial institution identification."""
    ClrSysMmbId: ClrSysMmbId
    Nm: Optional[str] = None
    PstlAdr: Optional[PstlAdr] = None


@dataclass
class InstgAgt:
    """Instructing agent."""
    FinInstnId: FinInstnId


@dataclass
class InstdAgt:
    """Instructed agent."""
    FinInstnId: FinInstnId


@dataclass
class DbtrAcct:
    """Debtor account."""
    Id: IdAcct


@dataclass
class CdtrAcct:
    """Creditor account."""
    Id: IdAcct


@dataclass
class Dbtr:
    """Debtor party."""
    Nm: str
    PstlAdr: PstlAdr


@dataclass
class Cdtr:
    """Creditor party."""
    Nm: str
    PstlAdr: PstlAdr


@dataclass
class DbtrAgt:
    """Debtor agent."""
    FinInstnId: FinInstnId


@dataclass
class CdtrAgt:
    """Creditor agent."""
    FinInstnId: FinInstnId



# SPDX-License-Identifier: Apache-2.0

"""Common payment-related models used across ISO20022 messages."""

from dataclasses import dataclass, field
from typing import Any, Dict, Optional


@dataclass
class PmtId:
    """Payment identification."""
    EndToEndId: str
    UETR: str
    InstrId: Optional[str] = None
    TxId: Optional[str] = None


@dataclass
class LclInstrm:
    """Local instrument."""
    Prtry: str = "CTRC"


@dataclass
class PmtTpInf:
    """Payment type information."""
    LclInstrm: LclInstrm


@dataclass
class SttlmInf:
    """Settlement information."""
    SttlmMtd: str = "CLRG"
    ClrSys: Dict[str, Any] = field(default_factory=lambda: {"Cd": "FDW"})


@dataclass
class GrpHdr:
    """Group header."""
    MsgId: str
    CreDtTm: str
    NbOfTxs: Optional[NbOfTxs] = None
    SttlmInf: Optional[SttlmInf] = None

@dataclass
class OrgnlGrpInf:
    """Original group information."""
    OrgnlMsgId: str
    OrgnlMsgNmId: str
    OrgnlCreDtTm: str



# SPDX-License-Identifier: Apache-2.0

from dataclasses import dataclass, asdict
from datetime import datetime, timezone
from typing import Any, Dict, Optional
from uuid import uuid4

from miso20022.common import (
    PstlAdr, ClrSysId, ClrSysMmbId, FinInstnId,
    InstgAgt, InstdAgt, GrpHdr, OrgnlGrpInf
)

@dataclass
class TxInf:
    """Transaction information."""
    OrgnlGrpInf: OrgnlGrpInf
    OrgnlInstrId: Optional[str] = None
    OrgnlEndToEndId: Optional[str] = None
    OrgnlUETR: Optional[str] = None
    InstgAgt: Optional[InstgAgt] = None
    InstdAgt: Optional[InstdAgt] = None


@dataclass
class FIToFIPmtStsReq:
    """Financial Institution to Financial Institution Payment Status Request."""
    GrpHdr: GrpHdr
    TxInf: TxInf


@dataclass
class Document:
    """PACS.028 document."""
    FIToFIPmtStsReq: FIToFIPmtStsReq

    def to_dict(self) -> Dict[str, Any]:
        """Convert this model to a dictionary representation for XML generation."""
        return {
            "Document": {
                "@xmlns:pacs": "urn:iso:std:iso:20022:tech:xsd:pacs.028.001.03",
                **asdict(self)
            }
        }

    @classmethod
    def from_payload(
        cls,
        payload: Dict[str, Any]
    ) -> "Document":
        """
        Create a payment status request document from payload data.
        
        Args:
            payload: A dictionary containing the payload data
            
        Returns:
            Document: A PACS.028 document
        """
        try:
            # Extract fedWireMessage data
            fed_wire_message = payload["fedWireMessage"]
            input_message_data = fed_wire_message["inputMessageAccountabilityData"]
            
            # Extract message ID components
            input_cycle_date = input_message_data["inputCycleDate"]
            input_source = input_message_data["inputSource"]
            input_sequence_number = input_message_data["inputSequenceNumber"]
            
            # Construct message ID
            msg_id = f"{input_cycle_date}{input_source}{input_sequence_number}"
            
            # Get pacs.028 specific fields
            original_msg_id = payload["original_msg_id"]
            original_msg_nm_id = payload["original_msg_nm_id"]
            original_creation_datetime = payload.get("original_creation_datetime")
            
            # Get optional fields
            original_instr_id = payload.get("original_instr_id")
            original_end_to_end_id = payload.get("original_end_to_end_id")
            original_uetr = payload.get("original_uetr")
            
            # Get sender/receiver information
            sender_info = fed_wire_message["senderDepositoryInstitution"]
            receiver_info = fed_wire_message["receiverDepositoryInstitution"]
            instg_agt_id = sender_info["senderABANumber"]
            instd_agt_id = receiver_info["receiverABANumber"]
                
            # Set default creation time to now if not provided
            if not original_creation_datetime:
                original_creation_datetime = datetime.now(timezone.utc).isoformat()
                
            # Create group header
            grp_hdr = GrpHdr(
                MsgId=msg_id,
                CreDtTm=datetime.now(timezone.utc).isoformat(),
                NbOfTxs=None, # Not needed for PACS.028
                SttlmInf=None  # Not needed for PACS.028
            )
            
            # Create original group info
            orgn_grp_inf = OrgnlGrpInf(
                OrgnlMsgId=original_msg_id,
                OrgnlMsgNmId=original_msg_nm_id,
                OrgnlCreDtTm=original_creation_datetime
            )
            
            # Create transaction info
            tx_inf = TxInf(
                OrgnlGrpInf=orgn_grp_inf,
                OrgnlInstrId=original_instr_id,
                OrgnlEndToEndId=original_end_to_end_id,
                OrgnlUETR=original_uetr
            )
            
            # Add instructing agent if provided
            if instg_agt_id:
                tx_inf.InstgAgt = InstgAgt(
                    FinInstnId=FinInstnId(
                        ClrSysMmbId=ClrSysMmbId(
                            ClrSysId=ClrSysId(),
                            MmbId=instg_agt_id
                        )
                    )
                )
                
            # Add instructed agent if provided
            if instd_agt_id:
                tx_inf.InstdAgt = InstdAgt(
                    FinInstnId=FinInstnId(
                        ClrSysMmbId=ClrSysMmbId(
                            ClrSysId=ClrSysId(),
                            MmbId=instd_agt_id
                        )
                    )
                )
            
            # Create FIToFIPmtStsReq
            fi_to_fi_pmt_sts_req = FIToFIPmtStsReq(
                GrpHdr=grp_hdr,
                TxInf=tx_inf
            )
            
            return cls(FIToFIPmtStsReq=fi_to_fi_pmt_sts_req)
        except KeyError as e:
            raise ValueError(f"Missing required key in payload: {e}")



# SPDX-License-Identifier: Apache-2.0

from dataclasses import dataclass, asdict
from datetime import datetime, timezone
from typing import Any, Dict, Optional, List
from uuid import uuid4

from miso20022.common import (
    PstlAdr, ClrSysId, ClrSysMmbId, FinInstnId,
    InstgAgt, InstdAgt, GrpHdr, OrgnlGrpInf
)

@dataclass
class Rsn:
    Prtry: str


@dataclass
class StsRsnInf:
    Rsn: Rsn
    AddtlInf: str
    

@dataclass
class TxInfAndSts:
    OrgnlGrpInf: OrgnlGrpInf
    TxSts: str
    InstgAgt: InstgAgt
    InstdAgt: InstdAgt
    StsId: Optional[str] = None
    OrgnlUETR: Optional[str] = None
    StsRsnInf: Optional[StsRsnInf] = None
    AccptncDtTm: Optional[str] = None
    FctvIntrBkSttlmDt: Optional[str] = None

    
    def __post_init__(self):
        """Validate that TxSts has one of the allowed values."""
        allowed_values = ["ACSC", "PDNG", "RJCT"]
        if self.TxSts and self.TxSts not in allowed_values:
            raise ValueError(f"TxSts must be one of {allowed_values}, got {self.TxSts}")

@dataclass
class FIToFIPmtStsRpt:
    GrpHdr: GrpHdr
    TxInfAndSts: TxInfAndSts

    @classmethod
    def from_iso20022(
        cls,
        data: Dict[str, Any]
    ) -> "FIToFIPmtStsRpt":
        
        # 1. Extract FIToFIPmtStsRpt data
        fi_to_fi_pmt_sts_rpt_data = data['FedwireFundsOutgoing']['FedwireFundsOutgoingMessage']['FedwireFundsPaymentStatus']['Document']['FIToFIPmtStsRpt']

        # 2. Instantiate the FIToFIPmtStsRpt data class
        pmt_sts_rpt = FIToFIPmtStsRpt(
            GrpHdr=GrpHdr(**fi_to_fi_pmt_sts_rpt_data['GrpHdr']),
            TxInfAndSts=TxInfAndSts(**fi_to_fi_pmt_sts_rpt_data['TxInfAndSts'])
        )
        
        return pmt_sts_rpt
   
